
import { Container, ContainerModule, interfaces } from 'inversify';
import { IFilesSettings, NATS_HEMERA_KUBE_FILESERVICE } from '@files-stack/core';
import WorkbenchModule from '@workbench-stack/module-server';
import FilesModule from '@files-stack/module-server';
import PreferencesModule from '@workbench-stack/preferences-module-server';
import { Feature } from '@common-stack/server-core';
import * as RPCHost from '@apollo-link-rpc/link-rpc-host';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { ServerTransport } from '../ws-transport';
import ApolloClient from 'apollo-client';
import { config } from '../config';
import { TYPES } from '@workbench-stack/core';
import { ClientTypes } from '@common-stack/client-core';
import {
    EnvironmentService, parseArgs, StorageService, InMemoryLocalStorage,
    IWorkspaceSettings, ICoreSettings,
} from '@workbench-stack/server-core';


const subTopic = `${config.NAMESPACE}/${config.CONNECTION_ID}`; // PrefernceUpdateHemera/filesServer/namespace/connection_id
const environmentService = new EnvironmentService(parseArgs(['--builtin-extensions-dir', '/tmp']), process.execPath);
const storageService = new StorageService(new InMemoryLocalStorage(), null, '');


type ISettings = IWorkspaceSettings & ICoreSettings & {
    // mongoConnection: any;
    // subTopic: string;
    // workspaceConfigFolderName: string;
    user: string;
};

export const settings: IFilesSettings & { name: string } & ISettings = {
    name: NATS_HEMERA_KUBE_FILESERVICE,
    rootFilePath: config.FILE_ROOT_PATH,
    connectionId: config.CONNECTION_ID,
    namespace: config.NAMESPACE,
    isEmptyWorkspace: true,
    subTopic,
    environmentService,
    storageService,
    user: '1',
    logger: config.LOG_LEVEL as any,
    workspaceId: config.CONNECTION_ID || 'DEFAULT',
    graphqlUrl: config.GRAPHQL_URL,
    configPath: process.env.CONFIG_PATH,
};

class UtilityClass {
    constructor(private modules) {
    }
    public getCacheKey(storeObj) {
        return this.modules.getDataIdFromObject(storeObj);
    }
}


const connectionManager = new ServerTransport();
const cache = new InMemoryCache();
const apolloClient = new ApolloClient({
    cache,
    link: new RPCHost.RPCLink({
        transport: new RPCHost.RpcHostInstance(connectionManager),
    }),
});

const defaultModule =
    () => new ContainerModule((bind: interfaces.Bind) => {
        bind('ApolloClient').toConstantValue(apolloClient);
        bind('ConnectionManager').toConstantValue(connectionManager);
        bind('Settings').toConstantValue(settings).whenTargetTagged('default', true);
        bind('Settings').toConstantValue(settings).whenTargetTagged('microservice', true);
        bind('MongoOptions').toConstantValue({});
        bind(TYPES.IExtensionService).toConstantValue({});
        bind(ClientTypes.ApolloClient).toConstantValue(apolloClient);
        bind(ClientTypes.InMemoryCache).toConstantValue(cache);

    });

const defaultServiceGen = (container: interfaces.Container) => ({
    apollo: container.get('ApolloClient'),
    connectionManager: container.get('ConnectionManager'),
});

const DefaultFeature = new Feature({
    createServiceFunc: defaultServiceGen,
    createContainerFunc: [defaultModule],
    createHemeraContainerFunc: [defaultModule],
});

export default new Feature(
    DefaultFeature,
    WorkbenchModule,
    PreferencesModule,
    FilesModule,
);



