import { IMessagePassingProtocol, Status } from '@apollo-link-rpc/common';
import { Socket }from 'net';

export class ServerTransport implements IMessagePassingProtocol<Socket> {
    private _connections = [];
    private _handlers = {
        open: [],
        close: [],
        error: [],
        message: [],
    };

    public onopen(fn) {  }
    public status = Status.Open;

    set onclose(fn) { this._handlers.close.push(fn); }
    set onerror(fn) { this._handlers.error.push(fn); }
    set onmessage(fn) { this._handlers.message.push(fn); }

    constructor() {
        this._handlers.open.forEach(fn => fn());
    }

    public connect(socket: Socket) {
        this._connections.push(socket);

        socket.on('error', () => this._handlers.error.forEach(fn => fn()));
        socket.on('message', (msg: string) => {
            this._handlers.message.forEach(fn => fn(JSON.parse(msg)));
        });
    }

    public disconnect(socket: Socket) {
        this._connections = this._connections.filter(ws => ws !== socket);
    }

    public close() {
        this._handlers.close.forEach(fn => fn());
        this._connections.forEach(ws => ws.close());
    }

    public send(arg: any) {
        this._connections.forEach(ws => {
            try {
                ws.send(JSON.stringify(arg));
            } catch (e) {
                console.log('Error throwed: ', e);
            }
        });
    }
}
